/*
 * JBoss, Home of Professional Open Source
 * 
 * Copyright 2010, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @authors tag. All rights reserved.
 * See the copyright.txt in the distribution for a full listing
 * of individual contributors.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License, v. 2.0.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License,
 * v. 2.0 along with this distribution; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */
package org.mobicents.slee.resource.diameter;

import net.java.slee.resource.diameter.base.DiameterActivity;

import org.mobicents.slee.resource.diameter.base.DiameterActivityHandle;

/**
 * 
 * Interface for activity management in diameter RAs. Implementation may store
 * activities locally or remotely.
 * 
 * @author <a href="mailto:baranowb@gmail.com"> Bartosz Baranowski </a>
 * @author <a href="mailto:brainslog@gmail.com"> Alexandre Mendonca </a>
 */
public interface DiameterActivityManagement {

  /**
   * Adds activity.
   * 
   * @param handle
   * @param activity
   */
  public void put(DiameterActivityHandle handle, DiameterActivity activity);

  /**
   * Retrieves activity from storage. If its present it performs any required operation 
   * 
   * @param handle
   * @return
   */
  public DiameterActivity get(DiameterActivityHandle handle);

  public DiameterActivity remove(DiameterActivityHandle handle);

  //public Set<DiameterActivityHandle> keySet();

  public boolean containsKey(DiameterActivityHandle activityHandle);

  public void update(DiameterActivityHandle handle, DiameterActivity activity);
}
